       program glshice2grib2

!*****************************************************************
!  prgmmr: Jerry           org: 1         date: 2006-07-05   *
!                                                                 *
!  abstract:                                                      *
!  use steve gilbert's gribcreate, addgrid, addfield, and gribend *
!  subroutines to write sourthern ice  files in Grib2 format     * 
!                                                                 *
!  program history log:                                           *
!    2006-07-05  Jerry Guo                                          *
!                                                                 *
!  input argument list:                                           *
!                                                                 *              
! 1. g(nx,ny,nflds): floating point array that contains all the   * 
!    nflds fields of dimension nx*ny each                         *
!                                                                 *
! 2. varname(nflds): character array that conatains the names of  *
!    all the nflds fields from g(nx,ny,nflds)                     *
!                                                                 *
! 2. typeproc(nflds): character array that contains process types *
!    (analysis, forecast, or analysis error)                      *
!                                                                 *
! 3. y0,m0,d0,h0,m1,s0: reference time (year, month, day, hour,   *
!    minutes and seconds)                                         *
!                                                                 *
! 4. reftime : significance of reference time. Values of interest to *
!    NDFD are 0 for analysis and 1 for start of forecast.         *
!                                                                 *
! 5. ityped: type of processed data. Values of interest to        *
!    NDFD are 0 for analysis products and 1 for forecast products *
!                                                                 *
!  output argument list:                                          *
! 1. filename: Name of output Grib2 file                          *
!                                                                 *
! attributes:                                                     *
!   language: f90                                                 *
!   machine:  LINUX                                               *
!******************************************************************
 
      implicit none

      integer(4) reftime ,y0,m0,d0,h0,m1,s0,ityped
      integer(4), parameter::nflds=2
!      integer(4), parameter::max_bytes=200*130000

!      integer(4), parameter::max_bytes=60751000
      integer(4), parameter::nx=18000
      integer(4), parameter::ny=4500
      integer(4) icharp, CtoI 
      integer(4) n,i,j,ij
      
      character*60 g2filename
      character*60 simfilename, arg_name(6), time_file_name
      character*60 varname(nflds)
      character*60 typeproc(nflds)
     
      character*1 sim(nx,ny), ice_time(nx,ny)
      
      real(4) g(nx,ny,2)
      real(4) simg2(nx,ny)
!      data simfilename/'/data/GLSNOW_2KM/SHEM/PRODUCT/SH.4YYDAYSIM'/
!      data wfrfilename/'/data/SNOW/SHSNOW/SHEM/SH.400000WFR'/
!      nflds=3
      call getarg(1, arg_name(1))
      simfilename=arg_name(1)
      print *, 'simfilename=', simfilename 
      
      call getarg(2, arg_name(2))
      time_file_name=arg_name(2)
      print *, 'time_file_name=', time_file_name

      call getarg(3, arg_name(3))
      g2filename=arg_name(3)
      call getarg(4, arg_name(4))
      call getarg(5, arg_name(5))
      call getarg(6, arg_name(6))

!      varname=(/"ice", "ice_time"/)
      varname(1) = "ice"
      varname(2) = "ice_time"
      typeproc=(/"observation", "observation"/)
      reftime=0
      y0=CtoI(arg_name(4))
      m0=CtoI(arg_name(5))
      d0=CtoI(arg_name(6))
      print *, 'Date:', y0, m0, d0


      h0=09
      m1=10
      s0=0
      ityped=0
!      g2filename="SH4km_autoice_07022006_grb"
      
!      simfilename="home/oper/grib2/fv/data/SH.406183SIM"
      open  (30, file=simfilename, access='direct',form='unformatted',status='old',recl=nx*ny)
      read  (30, rec=1) sim 
      close (30)  
      open  (31, file=time_file_name, access='direct',form='unformatted',status='old',recl=nx*ny)
      read  (31, rec=1) ice_time
      close (31)
      
      g=0.0
      do  i=1, nx
      do  j=1, ny
      simg2(i,j) = icharp(sim(i,j))
      g(i,j,2)=icharp(ice_time(i,j))*86400.0
      if (simg2(i,j) .eq. 3) then
          g(i,j,1) = 100
      endif
      enddo
      enddo

!      write (*,1110) simg2(800,1000)
! 1110 format (/F4.1)

       print*,'HERE I AM 2' 

      call write_ice_grib2(g,nx,ny,nflds,varname,typeproc, &
                 reftime ,y0,m0,d0,h0,m1,s0,ityped,g2filename)
       print*,'HERE I AM 3' 
      end program glshice2grib2    
   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine write_ice_grib2(g,nx,ny,nflds,varname,typeproc, & 
                 reftime ,y0,m0,d0,h0,m1,s0,ityped,filename)
      implicit none
!      integer(4), parameter::max_bytes=162000800
      integer(4), parameter::max_bytes=81000800*8
!      integer(4), parameter::max_bytes=81000800
!      integer(4), parameter::max_bytes=40500800
!      integer(4), parameter::max_bytes=200*130000
      integer(4), parameter::igdstmplen=19
      integer(4), parameter::idefnum=1
      integer(4), parameter::ipdstmplen=15
!      integer(4), parameter::ipdstmplen=5
      integer(4), parameter::idrstmplen=5
      integer(4) igdstmpl(igdstmplen) 
      integer(4) ideflist(idefnum)     
      integer(4) ipdstmpl(ipdstmplen)
      integer(4) idrstmpl(idrstmplen)
      integer(4) idrsnum,ibmap,numcoord,ipdsnum
      integer(4) lengrib,dscal
      integer(4) lunout,kwfr
      integer(4) nflds
      real(4) g(nx,ny,2)

      integer(4) nx,ny,reftime ,y0,m0,d0,h0,m1,s0,ityped
      integer(4) igds(5)
      integer(4) listsec0(2)
      integer(4) listsec1(13)
      real(4) fld(nx*ny)
      real(4) coordlist(1)
      integer(4) ierr,n,i,j,ij
      integer(4) icharp
      character*60 filename, wfrfilename
      character*60 varname(1)
      character*60 typeproc(1)
!      character*60 varname(2)
!      character*60 typeproc(2)
      character*1  cgrib(max_bytes)
      character*1 wfr(nx,ny)
      logical*1 bmap(nx,ny)
!      data wfrfilename/'/data/GLSNOW_2KM/SHEM/SH.400000WFR'/
      data wfrfilename/'SH.400000WFR'/
      
      open  (50, file=wfrfilename, access='direct',form='unformatted',status='old',recl=nx*ny)
      read  (50, rec=1) wfr
      close (50)

      bmap = .true.
      do j=1, ny
      do i=1, nx
       kwfr=icharp(wfr(i,j))
       if (kwfr .lt. 80) then
          bmap(i,j) = .false. ! land
       endif
      enddo
      enddo
! file that will store grib2 messages
      lunout=40

      print*,'about to open',lunout,trim(filename)
!
      call baopenw(lunout,trim(filename),ierr)

      print*,'write_ice_grib2:  opened ',lunout, &
             'for grib2 data:  ',trim(filename), &
             ' ;return code is ',ierr


!==>initialize new GRIB2 message and pack
! GRIB2 sections 0 (Indicator Section) and 1 (Identification 
! Section)

       listsec0(1)=10 ! Discipline-GRIB Master Table Number (see Code Table 0.0)
       listsec0(2)=2 ! GRIB Edition Number (currently 2)
       
       listsec1(1)=160       ! Id of orginating centre (Common Code Table C-1)
       listsec1(2)=0       ! Id of orginating sub-centre (local table)/Table C of ON388
       listsec1(3)=5       ! GRIB Master Tables Version Number (Code Table 1.0)
       listsec1(4)=0       ! GRIB Local Tables Version Number (Code Table 1.1)
       listsec1(5)=3    ! Significance of Reference Time (Code Table 1.2)
       listsec1(6)=y0      ! Reference Time - Year (4 digits)
       listsec1(7)=m0      ! Reference Time - Month
       listsec1(8)=d0      ! Reference Time - Day
       listsec1(9)=h0      ! Reference Time - Hour
       listsec1(10)=m1     ! Reference Time - Minute
       listsec1(11)=s0     ! Reference Time - Second
       listsec1(12)=1      ! Production status of data (Code Table 1.3)
       listsec1(13)=6 ! Type of processed data (Code Table 1.4)
        
       call gribcreate(cgrib,max_bytes,listsec0,listsec1,ierr)
       print*,'gribcreate status=',ierr

!==> Pack up Grid Definition Section (Section 3) add to GRIB2 message.

       igds(1)=0      !Source of grid definition (see Code Table 3.0)
       igds(2)=nx*ny  !Number of grid points in the defined grid.
       igds(3)=0      !Number of octets needed for each additional grid points definition
       igds(4)=0      !Interpretation of list for optional points definition (Code Table 3.11)
       igds(5)=0     !Grid Definition Template Number (Code Table 3.1)

       call apply_template_30_ice(igdstmpl,igdstmplen) 

       ideflist=0     !Used if igds(3) .ne. 0. Dummy array otherwise

       call addgrid(cgrib,max_bytes,igds,igdstmpl,igdstmplen,ideflist,idefnum,ierr)
       print*,'addgrid status=',ierr

       do n= 1,nflds
 
!==> pack up sections 4 through 7 for a given field and add them to a GRIB2 message.  
! They are Product Definition Section, Data Representation Section, Bit-Map Section 
! and Data Section, respectively.

       ipdsnum=30    !Product Definition Template Number ( see Code Table 4.0)

       print*,'HERE I AM 1' 
       print*,'write_ice_grib2:, n,varname,typeproc=', & 
                                n,varname(n),typeproc(n)

       call apply_template_40_ice(ipdstmpl,ipdstmplen, & 
                                varname(n),typeproc(n))

       numcoord=0
       coordlist=0. !needed for hybrid vertical coordinate
       idrsnum=0    !Data Representation Template Number ( see Code Table 5.0 )

       call apply_template_50_ice(idrstmpl,idrstmplen,varname(n))
       dscal=idrstmpl(3) ! see template 50

!       print*, 'before fid'
       ij=0 
       do j=1,ny
       do i=1,nx
          ij=ij+1
          fld(ij)=g(i,j,n)
!       fld(ij)=0.0
       enddo
       enddo

!       print*, 'after fid'

       ibmap=0     ! Bitmap indicator ( see Code Table 6.0 ),  
!       print*, 'before addfield'
       call addfield(cgrib,max_bytes,ipdsnum,ipdstmpl,ipdstmplen, &
                          coordlist,numcoord,idrsnum,idrstmpl, &
                          idrstmplen,fld,nx*ny,ibmap,bmap,ierr)
       print*,'addfield status=',ierr
      enddo 

!==> finalize  GRIB2 message after all grids
! and fields have been added.  It adds the End Section ( "7777" )

       call gribend(cgrib,max_bytes,lengrib,ierr)
       print*,'gribend status=',ierr
       print*,'length of the final GRIB2 message in octets =',lengrib
       call wryte(lunout, lengrib, cgrib)

      return
      end subroutine
!===========================================================================
       subroutine apply_template_30_ice(ifield3,len3) 

       implicit none

!       integer(4),parameter::nx=9000
!       integer(4),parameter::ny=2250
       integer(4),parameter::nx=18000
       integer(4),parameter::ny=4500
       integer(4),parameter::lat1=00000000    !lat of 1st grd pt in micro-deg
       integer(4),parameter::lon1=180000000  !east-long of 1st grd pt in micro-deg 
!      integer(4),parameter::lat1=20192000   !lat of 1st grd pt in micro-deg
!      integer(4),parameter::lon1=238446000  !east-long of 1st grd pt in micro-deg 
       integer(4),parameter::latan1=25000000 !true lat in micro-deg 
       integer(4),parameter::lonv=265000000  !y-axis || to long-circle at this long
!       integer(4),parameter::ds1=255079406     !grid spacing in x and y used for 4km
       integer(4),parameter::ds1=127539703     !grid spacing in x and y used for 2km
!      integer(4),parameter::ds1=5079000     !grid spacing in x and y

       integer(4) len3
       integer(4) ifield3(len3)                  
                                                                        
       ifield3(1) = 0 !Earth assumed spherical with radius of 6,367,470.0 m
       ifield3(2) = -1
       ifield3(3) = -1
       ifield3(4) = -1
       ifield3(5) = -1
       ifield3(6) = -1
       ifield3(7) = -1
       ifield3(8) = nx ! 18000
       ifield3(9) = ny ! 4500
       ifield3(10) =1 !not sure
       ifield3(11) = 1000000 !not sure 
       ifield3(12) = -20000.0 
       ifield3(13) = 180020000.0 ! 
       ifield3(14) = 48!   00001000 bits not sure 
       ifield3(15) = -89980000.0
       ifield3(16) = 179980000.0
       ifield3(17) = 20000.0
       ifield3(18) = 20000.0
       ifield3(19) = 0 ! 01000000 bits not sure

       
!       ifield3(1) = 0 !Earth assumed spherical with radius of 6,367,470.0 m
!       ifield3(2) = 0
!       ifield3(3) = 0
!       ifield3(4) = 0
!       ifield3(5) = 0
!       ifield3(6) = 0
!       ifield3(7) = 0
!       ifield3(8) = nx ! 9000
!       ifield3(9) = ny ! 2250
!       ifield3(10) =1 !not sure
!       ifield3(11) = 1000000 !not sure 
!       ifield3(12) = 0.0 
!       ifield3(13) = 0.0 ! 
!       ifield3(14) = 8!   00001000 bits not sure 
!       ifield3(15) = 90000000.0
!       ifield3(16) = 360000000.0
!       ifield3(17) = 40000.0
!       ifield3(18) = 40000.0
!       ifield3(19) = 0 ! 11000000 bits not sure
       return
       end
!===========================================================================
       subroutine apply_template_40_ice(ifield4,len4,var,typep) !using template 4.30
       
       implicit none


       integer(4) len4
       integer(4) ifield4(len4)
   
       character*60 var
       character*60 typep
      
       ifield4=0 
!==> ifield4(1):parameter category (see Code Table 4.1)
!==> ifield4(2):parameter number (see Code Table 4.2)

       if (trim(var) .eq. 'ice' ) then
         ifield4(1) = 2  ! Octet10 in Section4, and Octet7 in section0 should be 0
         ifield4(2) = 0 ! Octet11 in Section4.
       elseif (trim(var) .eq. 'ice_time' ) then
         ifield4(1) = 191 ! (191=Miscellaneous) here is Octet10 in section4, and  Octet7 in section0 should be 0
         ifield4(2) = 0 ! (0=Seconds prior to initial reference time ) Octet11 in Section4.
       else
         print*,'define param category and param number for field ',var
         print*,'sorry, ... aborting in apply_template_40_ice' 
         call abort
       endif 

!==> ifield4(3):type of generating process (see Code Table 4.3)
       if (trim(typep) .eq. 'observation') then
         ifield4(3) = 8
        else
         print*,'for ice, typep must be observation' 
         print*,'sorry, ... aborting in apply_template_40_ice' 
         call abort
       endif

!==> ifield4(4):background generating process identifier 
!                         (defined by originating Center)
       ifield4(4) = 0 !hasn't been defined yet 

!==> ifield4(5):analysis or forecast generating process identifier 
!                         (defined by originating Center)
       ifield4(5) = 0 !Snow Interpretationn value in Output Files

!==> ifield4(6):hours of observational data  
!==> ifield4(7):minutes of observational data  

!       ifield4(6) = 23   !per Rusty
!       ifield4(7) = 7   

!==> ifield4(8):indicator of unit of time range (see Code Table 4.4) 
!       ifield4(8) = 2    !run once daily    

!==> ifield4(9):forecast time in units defined by ifield4(8) 
!       ifield4(9) = 0    !not sure


!       ifield4(11) = 1 !Ground or water surface, not sure
!
!       if (trim(var) .eq. 'ice') then
!         ifield4(10) = 1   !not sure
!         ifield4(12) = 0   !not sure
!       elseif (trim(var) .eq. 'ice' ) then
!         ifield4(10) = 1
!         ifield4(12) = 0
!       else
!         ifield4(10) = 1
!         ifield4(12) = 0 
!       endif

!==> ifield4(13):type of second fixed surface(See Code Table 4.5)
!==> ifield4(14):scale factor of second fixed surface
!==> ifield4(15):scaled value of second fixed surface

!       ifield4(13) = 1 !not sure
!       ifield4(14) = 0   !not sure
!       ifield4(15) = 0   !not sure

       return
       end

!===========================================================================
       subroutine apply_template_50_ice(ifield5,len5,var) 

!********************************************************************************
! ifield5(1): reference value(R) (IEEE 32-bit floating-point value)             *
! ifield5(2): binary scale factor (E)                                           *
! ifield5(3): decimal scale factor (D)                                          *
! ifield5(4): number of bits used for each packed value for simple packing      *
!             or for each group reference value for complex packing or          *
!             spatial differencing                                              *
! ifield5(5): type of original field values (See Code Table 5.1)                *
!********************************************************************************
       implicit none

       integer(4) len5
       integer(4) ifield5(len5)
       character*60 var

       ifield5=0 ! Initialize

       if (trim(var) .eq. 'ice') then
!         ifield5(3) = 1  ! 2  should be 20
        elseif (trim(var) .eq. 'ice_time') then
!         ifield5(3) = 1  ! 3 should be 30
       else
         print*,'please define decimal scale factor ',var
         print*,'sorry, ... aborting in apply_template_50_ndfd'
         call abort
        endif

       ifield5(4) = 0 !not sure 
       ifield5(5) = 0 !Folating
      
       return
       end

!===========================================================================
       FUNCTION ICHARP(A)
      CHARACTER*1 A
        ICHARP=ICHAR(A)
        IF (ICHARP.LT.0) ICHARP=ICHARP+256
      RETURN
      END
      
      function CtoI(int_as_chars)
  
! Subroutine to convert a character string containing
! digits to an integer.
          
      Character(len=5)   int_as_chars
      Integer            CtoI

      Read (int_as_chars, FMT = '(I5)') CtoI
    
      return 
      end


